@@ -3,6 +3,7 @@
 
 #define _FILE_OFFSET_BITS 64
 
+#include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -1371,7 +1372,7 @@ static void LZe_full_flush(CLzmaEnc *p, uint32_t posState)
     unsigned long long in_size = p->nowPos64;
     unsigned long long out_size = p->rc.processed + Fh_size + Ft_size;
     if( in_size <= 0 || out_size <= 0 )
-      fprintf( stderr, " no data compressed.\n" );
+      fputs( " no data compressed.\n", stderr );
     else
       fprintf( stderr, "%6.3f:1, %6.3f bits/byte, "
                        "%5.2f%% saved, %llu in, %llu out.\n",
