@@ -1,5 +1,5 @@
 /*  Pdlzip - LZMA lossless data compressor
-    Copyright (C) 2010-2015 Antonio Diaz Diaz.
+    Copyright (C) 2010-2016 Antonio Diaz Diaz.
 
     This program is free software. Redistribution and use in source and
     binary forms, with or without modification, are permitted provided
@@ -17,11 +17,6 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
 
-#ifndef __cplusplus
-enum Bool { false = 0, true = 1 };
-typedef enum Bool bool;
-#endif
-
 #ifndef max
   #define max(x,y) ((x) >= (y) ? (x) : (y))
 #endif
@@ -34,9 +29,10 @@ typedef int State;
 enum {
   min_dictionary_bits = 12,
   min_dictionary_size = 1 << min_dictionary_bits,
-  max_dictionary_bits = 29,
+  max_dictionary_bits = 27,			/* kDicLogSizeMaxCompress */
   max_dictionary_size = 1 << max_dictionary_bits,
   literal_context_bits = 3,
+  literal_pos_state_bits = 0,				/* not used */
   pos_state_bits = 2,
 
   len_low_bits = 3,
@@ -61,7 +57,8 @@ struct Pretty_print
   };
 
 static inline void Pp_init( struct Pretty_print * const pp,
-                   const char * const filenames[], const int num_filenames )
+                            const char * const filenames[],
+                            const int num_filenames, const int verbosity )
   {
   unsigned stdin_name_len;
   int i;
@@ -71,6 +68,7 @@ static inline void Pp_init( struct Pretty_print * const pp,
   pp->first_post = false;
   stdin_name_len = strlen( pp->stdin_name );
 
+  if( verbosity <= 0 ) return;
   for( i = 0; i < num_filenames; ++i )
     {
     const char * const s = filenames[i];
@@ -121,6 +119,11 @@ static inline void CRC32_update_buf( uint32_t * const crc,
   }
 
 
+static inline bool isvalid_ds( const unsigned dictionary_size )
+  { return ( dictionary_size >= min_dictionary_size &&
+             dictionary_size <= max_dictionary_size ); }
+
+
 static inline int real_bits( unsigned value )
   {
   int bits = 0;
@@ -142,6 +145,14 @@ static inline void Fh_set_magic( File_header data )
 static inline bool Fh_verify_magic( const File_header data )
   { return ( memcmp( data, magic_string, 4 ) == 0 ); }
 
+/* detect truncated header */
+static inline bool Fh_verify_prefix( const File_header data, const int size )
+  {
+  int i; for( i = 0; i < size && i < 4; ++i )
+    if( data[i] != magic_string[i] ) return false;
+  return ( size > 0 );
+  }
+
 static inline uint8_t Fh_version( const File_header data )
   { return data[4]; }
 
@@ -158,21 +169,18 @@ static inline unsigned Fh_get_dictionary_size( const File_header data )
 
 static inline bool Fh_set_dictionary_size( File_header data, const unsigned sz )
   {
-  if( sz >= min_dictionary_size && sz <= max_dictionary_size )
+  if( !isvalid_ds( sz ) ) return false;
+  data[5] = real_bits( sz - 1 );
+  if( sz > min_dictionary_size )
     {
-    data[5] = real_bits( sz - 1 );
-    if( sz > min_dictionary_size )
-      {
-      const unsigned base_size = 1 << data[5];
-      const unsigned fraction = base_size / 16;
-      int i;
-      for( i = 7; i >= 1; --i )
-        if( base_size - ( i * fraction ) >= sz )
-          { data[5] |= ( i << 5 ); break; }
-      }
-    return true;
+    const unsigned base_size = 1 << data[5];
+    const unsigned fraction = base_size / 16;
+    int i;
+    for( i = 7; i >= 1; --i )
+      if( base_size - ( i * fraction ) >= sz )
+        { data[5] |= ( i << 5 ); break; }
     }
-  return false;
+  return true;
   }
 
 
